/*************************************************************** 
* 
* MemFitsReaderwriterX.cc     & XMM Science Analysis System (SAS) 
* 
* Class representing Fits File reader/writer.
* 
* (c) 1997 ESA - Written by Mark Thomas, CARA
*                and Marco Beijersbergen
* 
* $Id: MemFitsReaderWriterX.cc,v 1.6 2000/11/09 16:20:52 mthomas Exp $
* 
*****************************************************************/ 


//#include <string>
//#include <iostream.h>
#include <fitsio.h>
//#include <unistd.h>
//#include <strstream.h>

//#include <stringutil.h>
#include <Tracer.h>
#include <errstr.h>
#include <Dal.h>

//#include "utilities.h"
#include "MemFitsReaderWriterX.h"
#include "dal_utilities.h"


// ********************
// Global defined here.
// ********************

//static const string nameOfPrimaryImage = "PRIMARY";
//static const string defaultArrayComment = "The name of this array";
//static const string defaultTableComment = "The name of this table";


/*
int
MemFitsReaderWriterX::readColumn(
	fitsfile *	fptr,
	int &		columnNumber,
	Table &		table,
	int &		status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	char			name[ 32 ];
	int			typeCode;
	char			comment[ 1024 ];
	char			columnComment[ 1024 ];
	char			units[ 32 ];
	//long			repeat;
	//long			width;
	string			tableName;
	const unsigned int	maxdim = 4;
	int			naxis;
	long			naxes[ maxdim ];
	string			type = "TTYPE";
	Column::DataType 	dataType;
	Column * 		column;


	// *************************************
	// Get the column name & column comment.
	// *************************************
	fits_read_key(
			fptr,
			TSTRING,
			(char *)( type + toString( columnNumber + 1 )).c_str(),
			name,
			columnComment,
			&status );


	// *******************************
	// Read the optional column units.
	// *******************************
	readColumnUnits( fptr, columnNumber, units, comment, status );


	// ***************
	// Get dimensions.
	// ***************
	fits_read_tdim(
				fptr,
				columnNumber + 1,
				maxdim,
				&naxis,
				naxes,
				&status );

	fits_check(fptr);

	SAS_DEBUG( for (int i = 0; i < naxis; ++i) clog << "naxis["<<i<<"]="<<naxes[i]<<endl; )


	// *************************
	// Get the column data type.
	// *************************
	getColumnDataType(
		fptr,
		columnNumber,
		typeCode,
		//repeat,
		//width,
		dataType,
		status );

	if( !isValidDalDataType( abs( typeCode ) ))
	{
		errstr << "An invalid column data type has been found, and is being ignored" << Sas::message( Msg::Lib, Msg::Verbose );

		// ******************************
		// Everything is Ok, so return 0.
		// ******************************
		RETURN 0;
	} // if

	checkAndSetUnsignedColumn(
			fptr,
			columnNumber,
			dataType,
			status );

	// ********************************
	// Now add the column to the table.
	// ********************************

	if( typeCode >= 0 )
	{
		column = table.addColumn(
					name,
					dataType,
					columnComment,
					units,
					vector<unsigned long>(
						(unsigned long *)(naxes),
						(unsigned long *)(naxes + naxis) ));
	}
	else
	{
		column = table.addColumn(
					name,
					dataType,
					columnComment,
					units,
					Dal::zero );

	}// if


	SAS_DEBUG( clog << "added column " << column->name() << " to table " << table.name() << " type=" << typeCode << " dataType=" << int( dataType ) << " rows=" << column->rows() << " elements=" << column->elements() << " dimensions=" << column->dimensions() << endl; )

	// ***********************
	// Read Column Attributes.
	// ***********************
	ColumnKeywordChecker * cac = new ColumnKeywordChecker( column );

	readAttributes(
			fptr,
			(Attributable &)*column,
			cac,
			status );

	delete cac;

	// ***********************
	// TEMPORARILY DISABLED:
	// Get the Scaling Factors
	// ***********************
	//readColumnScalingFactors(
	//	fptr,
	//	columnNumber,
	//	*column,
	//	status );

	// **************************
	// Get the column null value.
	// **************************
	readColumnNullValue(
		fptr,
		*column,
		status );

	// ********************
	// Get the column data.
	// ********************
	if( typeCode >= 0 )
	{
		readFixedLengthColumnData( fptr, column, columnNumber, status );
	}
	else
	{
		readVariableLengthColumnData( fptr, column, columnNumber, status );
	} // if


	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;

}

*/

/*
int
MemFitsReaderWriterX::readTable(
	fitsfile * 	fptr,
	DataSet &	dataSet,
	int &		status ) const
{
	PROC

	if( status != 0 )
	{
		ALTRETURNV( 1 );
	} // if

	TableInfo	tableInfo;

	readTableInfo( fptr, tableInfo, status );

	// ******************************
	// Add the table to the data set.
	// ******************************
	Table* table = addTable( dataSet, tableInfo );

	//Table *		table = dataSet.addTable(
	//				tableInfo._name,
	//				tableInfo._rows,
	//				tableInfo._label );
//
	//errstr << "Added table with qualified name '" << table -> qualifiedName() << "'" << Sas::message( Msg::Lib, Msg::Verbose );

	// *********************
	// Add table attributes.
	// *********************
	const unsigned int tableAttributes( tableInfo._attributes.size() );

	for( unsigned int i = 0; i < tableAttributes; ++i )
	{
		AttributeInfo	ai( tableInfo._attributes[ i ] );
		Attribute* 	attribute( table -> addAttribute( ai._name, ai._label, ai._units ));

		setAttributeValue( attribute, ai._value );

		errstr << "Added attribute with qualified name '" << attribute -> qualifiedName() << "'" << Sas::message( Msg::Lib, Msg::Verbose );
	} // for

	const unsigned int columns( tableInfo._columns.size() );

	for( unsigned int i = 0; i < columns; ++i )
	{
		ColumnInfo	ci( tableInfo._columns[ i ] );
		Column*		column;

		if( !isValidDalDataType( abs( ci._typeCode ) ))
		{
			errstr << "An invalid column data type has been found, and is being ignored" << Sas::message( Msg::Lib, Msg::Verbose );

			continue;
		} // if

		ci._dataType = fromFITSDataType( abs( ci._typeCode ));
		checkAndSetUnsignedColumn( ci );

		column = table -> addColumn(
					ci._name,
					ci._dataType,
					ci._label,
					ci._units,
					ci._size );

		if( ci._nullValueFound )
		{
			column -> nullValue( ci._nullValue );
		} // if

		errstr << "Added column with qualified name '" << column -> qualifiedName() << "'. type = " << toString( column -> dataType() ) << " rows = " << column -> rows() << " elements = " << column -> elements() << " dimensions = [" << column -> dimensions() << "]" << Sas::message( Msg::Lib, Msg::Verbose );

		// *********************
		// Add column attributes.
		// *********************
		const unsigned int columnAttributes( ci._attributes.size() );

		for( unsigned int i = 0; i < columnAttributes; ++i )
		{
			AttributeInfo	ai( ci._attributes[ i ] );
			Attribute*	attribute( column -> addAttribute( ai._name, ai._label, ai._units ));

			setAttributeValue( attribute, ai._value );

			errstr << "Added attribute with qualified name '" << attribute -> qualifiedName() << "'" << Sas::message( Msg::Lib, Msg::Verbose );
		} // for
	} // for

	// ******************************
	// Everything is Ok, so return 0.
	// ******************************
	RETURN 0;

}
*/


bool
MemFitsReaderWriterX::acceptTableKeyword(
			fitsfile*	fptr,
			const string&	keyName ) const
{
	PROC
	//bool	accept = true;
	// ************************************************
	// The following keywords will be accepted quietly.
	// ************************************************
	if( keyName == "HDUNAME" )
	{
		//accept = true;
		RETURN true;
	}
	//else
	//{
		//RETURN FitsReaderWriter::acceptTableKeyword( fptr, keyName );
	//} // if
	else if( isFITSReservedKeyword( keyName ) )
	{
		errstr << "The reserved keyword '" << keyName << "' was found in a table extension in the fits file with name '" << fits_file_name( fptr ) << "', but is not allowed in this context. This keyword is being ignored." << Sas::warning( "reservedKeywordIgnored" );

		RETURN false;
	} // if
	//RETURN accept;
	//RETURN true;
	RETURN false;
}


FitsReaderWriter::Status
MemFitsReaderWriterX::columnDataTypeStatus( int typeCode ) const
{
	PROC
	Status		status = FitsReaderWriter::Good;

	if( abs( typeCode ) != TBIT && !isValidDalDataType( abs( typeCode ) ))
	{
		errstr << "An invalid column data type has been found, and is being ignored" << Sas::message( Msg::Lib, Msg::Verbose );

		status = FitsReaderWriter::Ignore;
	} // if

	RETURN status;
}


void
MemFitsReaderWriterX::setColumnDataType( ColumnInfo& columnInfo ) const
{
	PROC
	if( columnInfo._typeCode == TBIT )
	{
		//char*	format;

		errstr << "TBIT Column found: repeat = " << columnInfo._repeat << ", width = " << columnInfo._width << Sas::message( Msg::Core, Msg::Sparse );

		if( columnInfo._repeat < 9 )
		{
			columnInfo._repeat = 1;
		//	format = "1B";
		}
		else if( columnInfo._repeat < 17 )
		{
			columnInfo._repeat = 2;
		//	format = "1I";
		}
		else
		{
			columnInfo._repeat = 4;
		//	format = "1J";
		} // if
		//char	tform[ 12 ];
		//sprintf( tform, "TFORM%d", columnInfo._columnNumber );
		//fits_write_key_str(
		//	columnInfo._fptr,
		//	tform,
		//	format,
		//	"THIS USED TO BE AN X COLUMN",
		//	&status );
		//fits_check( columnInfo._fptr );

		columnInfo._size = Dal::scalar;

		errstr << "The TBIT column with name '" << columnInfo._name << "' has been encountered; this will be converted to an int32 column" << Sas::warning( "bitColumnEncountered" );
		//columnInfo._repeat = 1;
		columnInfo._dataType = Column::Int32;
		errstr << "TBIT Column modifed: repeat = " << columnInfo._repeat << ", width = " << columnInfo._width << Sas::message( Msg::Core, Msg::Sparse );
	}
	else
	{
		FitsReaderWriter::setColumnDataType( columnInfo );
	} // if
	ENDPROC
}


Table*
MemFitsReaderWriterX::addTable(
			DataSet&				dataSet,
			TableInfo& 				tableInfo ) const
{
	PROC
	const unsigned int	blockNumber( dataSet.blocks() );

	// ******************************
	// Add the table to the data set.
	// ******************************
	if( tableInfo._name.empty() )
	{
		errstr << "A fits table was found without a name." << Sas::warning( "unnamedTable" );

		tableInfo._name = "NONAME";
	}
	else
	{
		// ***************************************
		// Check to see if the HDUNAME is present.
		// ***************************************
		char			hduname[ 24 ];
		char 			comment[ 1024 ];

		fits_read_keyword(
				tableInfo._fptr,
				"HDUNAME",
				hduname,
				comment,
				&status );

		if( status && status != KEY_NO_EXIST ) fits_check( tableInfo._fptr );
		status = 0;

		errstr << "The fits keyword 'HDUNAME' with value '" << hduname << "' was found in the table extension with name '" << tableInfo._name << "'. This table extension name will be discarded and replaced by '" << hduname << "'" << Sas::warning( "hdunameFound" );

		tableInfo._name = hduname;
	} // if

	// *****************************************************
	// If the name already existed, append the block number.
	// *****************************************************
	if ( dataSet.hasBlock( tableInfo._name ) )
	{
		errstr << "A duplicate block name '" << tableInfo._name << "' has been encountered. The block name '";
		tableInfo._name += toString( blockNumber );
		errstr << tableInfo._name << "' will be used instead." << Sas::warning( "duplicateTableName" );

	} // if

	Table *		table = dataSet.addTable(
					tableInfo._name,
					tableInfo._rows,
					tableInfo._label );

	errstr << "Added table with qualified name '" << table -> qualifiedName() << "'" << Sas::message( Msg::Core, Msg::Verbose );

	RETURN table;
}


