/*
	Automatically generated source file template for a task.
	(c) European Space Agency


	Title: dsmv
	Package: daltools
	Author: Mark Thomas
	Affiliation: CARA/ESA
	Email: mthomas@astro.estec.esa.nl
	Purpose: Move an object
	Description: Move an object


*/

#include <Tracer.h>
#include <param.h>
#include <Dal.h>
#include <CompoundName.h>



#include "dsmv.h"

void dsmv()
{
	PROC
	const string		from( stringParameter( "from" ) );
	const string		to( stringParameter( "to" ) );
	const CompoundName	fcn( from, CompoundName::Any, 0, Dal::Modify );
	const CompoundName	tcn( to, CompoundName::Any, 0, Dal::Modify );
	const bool		withPosition( booleanParameter( "withposition" ) );
	int			position( intParameter( "position" ));

	switch( fcn.state() )
	{
		case CompoundName::DataSetName :

			errstr << "Dataset type '" << from << "' not yet supported" << Sas::error( "unsupportedObjectType" );

			break;
		case CompoundName::BlockName :
		{
			if( tcn.state() == CompoundName::BlockName )
			{
				if( fcn.dataSetName() == tcn.dataSetName() )
				{
					Block* srcBlock( fcn.block() );
					srcBlock -> rename( tcn.blockName() );
				}
				else
				{
					Block* srcBlock( fcn.block() );
					DataSet* dtnDataSet( tcn.dataSet() );
					if( withPosition )
					{
						dtnDataSet -> add( srcBlock, tcn.blockName(), position );
					}
					else
					{
						dtnDataSet -> add( srcBlock, tcn.blockName() );
					} // if
					srcBlock -> parent() -> deleteBlock( srcBlock -> name() );
				} // if
			}
			else if( tcn.state() == CompoundName::DataSetName )
			{
				Block* srcBlock( fcn.block() );
				DataSet* dtnDataSet( tcn.dataSet() );
				if( withPosition )
				{
					dtnDataSet -> add( srcBlock, srcBlock-> name(), position );
				}
				else
				{
					dtnDataSet -> add( srcBlock, srcBlock-> name() );
				} // if
				srcBlock-> parent() -> deleteBlock( srcBlock -> name() );
			}
			else
			{
				errstr << "Cannot move block '" << from << "' to '" << to << "'" << Sas::error( "moveNotPossible" );
			} // if
		}
			break;
		case CompoundName::ColumnName :
		{
			if( tcn.state() == CompoundName::ColumnName )
			{
				if( fcn.qualifiedBlockName() == tcn.qualifiedBlockName() )
				{
					Column* srcColumn( fcn.column() );
					srcColumn -> rename( tcn.columnName() );
				}
				else
				{
					Column* srcColumn( fcn.column() );
					Table* dtnTable( tcn.table() );
					if( withPosition )
					{
						dtnTable -> add( srcColumn, tcn.columnName(), position );
					}
					else
					{
						dtnTable -> add( srcColumn, tcn.columnName() );
					} // if
					srcColumn -> parent() -> deleteColumn( srcColumn -> name() );
				} // if
			}
			else if( tcn.state() == CompoundName::BlockName )
			{
				Column* srcColumn( fcn.column() );
				Table* dtnTable( tcn.table() );
				if( withPosition )
				{
					dtnTable -> add( srcColumn, srcColumn -> name(), position );
				}
				else
				{
					dtnTable -> add( srcColumn );
				} // if
				srcColumn -> parent() -> deleteColumn( srcColumn -> name() );
			}
			else
			{
				errstr << "Cannot move column '" << from << "' to '" << to << "'" << Sas::error( "moveNotPossible" );
			} // if
		}
			break;
		case CompoundName::DataSetAttributeName :
		case CompoundName::BlockAttributeName :
		case CompoundName::ColumnAttributeName :
		{
			if(
				tcn.state() == fcn.state()
				|| tcn.state() != fcn.state()
				&& ( tcn.state() == CompoundName::DataSetAttributeName
				|| tcn.state() == CompoundName::BlockAttributeName
				|| tcn.state() == CompoundName::ColumnAttributeName ))
			{
				if( fcn.qualifiedAttributableName() == tcn.qualifiedAttributableName() )
				{
					Attribute* srcAttribute( fcn.attribute() );
					srcAttribute-> rename( tcn.attributeName() );
				}
				else
				{
					Attribute* srcAttribute( fcn.attribute() );
					Attributable* dtnAttributable( tcn.attributable() );
					Attribute* dtnAttribute = dtnAttributable -> addAttribute( srcAttribute );
					dtnAttribute -> rename( tcn.attributeName() );
					srcAttribute -> parent() -> deleteAttribute( fcn.attributeName() );
				} // if
			}
			//else if(
			//	tcn.state() == CompoundName::DataSetAttributeName
			//	|| tcn.state() == CompoundName::BlockAttributeName
			//	|| tcn.state() == CompoundName::ColumnAttributeName )
			//{
			//	Attribute* srcAttribute( fcn.attribute() );
			//	Attributable* dtnAttributable( tcn.attributable() );
			//	Attribute* dtnAttribute = dtnAttributable -> addAttribute( srcAttribute );
			//	dtnAttribute -> rename( tcn.attributeName() );
			//	srcAttribute -> parent() -> deleteAttribute( fcn.attributeName() );
			//}
			else if(
				tcn.state() == CompoundName::DataSetName
				|| tcn.state() == CompoundName::BlockName
				|| tcn.state() == CompoundName::ColumnName )
			{
				Attribute* srcAttribute( fcn.attribute() );
				Attributable* dtnAttributable( tcn.attributable() );
				dtnAttributable -> addAttribute( srcAttribute );
				srcAttribute -> parent() -> deleteAttribute( fcn.attributeName() );
			}
			else
			{
				errstr << "Cannot move attribute '" << from << "' to '" << to << "'" << Sas::error( "moveNotPossible" );
			} // if
		}
			break;
		default :
			errstr << "Object type '" << from << "' not supported" << Sas::error( "unsupportedObjectType" );
	} // switch

	ENDPROC
}
